#!/bin/csh
# A QTCM  ensemble run with prescribed SST (seasonal)
#
# ============================================================

# Set Macro switches for compilation (see makefile)
# -------------------------------------------------
#
set DEFS = 

# String to document the macros in effect
set defs_str = `echo "$DEFS" | sed -e 's/ -D/_/g' `

# adjust RUNNAME 
set RUNNAME = `basename $0 .sh`

# Set directories 
# ---------------
#
set SCRIPTDIR = `pwd`                    # directory of the script
set QTCMROOT  = $SCRIPTDIR/..            # root directory
set INIDIR    = $QTCMROOT/inidata        # directory of restart file
set SRCDIR    = $QTCMROOT/src            # Set the source directory 
set BNDDIR    = $QTCMROOT/bnddata/r64x42 # Boundary data directory
set OUTDIR    = $QTCMROOT/proc/${RUNNAME}${defs_str}  # Output dirctory
set LOGFILE = ${RUNNAME}.log             # model output log file
set SCRIPT  = `basename $0`              # script name 

# Let the user know the settings:
#
echo Script Settings:
echo Output directory: $OUTDIR
echo RUNNAME : "$RUNNAME"
echo Preprocessor macro definitions: $DEFS
echo -----------------------
echo ' '


# Compile qtcm in source dir
# --------------------------
#
cd $SRCDIR
# remove old object files if preprocessor flags are set 
if ( "$DEFS" != '' )  make clean 
make DEFS="${DEFS}" qtcm || exit


if ( ! -d $OUTDIR )  mkdir $OUTDIR

# Store script and source code in an archive in the output dir
#
tar -uf $OUTDIR/${RUNNAME}_src.tar *.f90 makefile
cd $SCRIPTDIR
tar -uf $OUTDIR/${RUNNAME}_src.tar $SCRIPT
gzip -f $OUTDIR/${RUNNAME}_src.tar

cd $OUTDIR

# Get restart file and executable
# ----------------------------------------------
# 
cp $INIDIR/qtcm.restart .
cp -fp $SRCDIR/qtcm .


# Copy the script into the output log file
# ----------------------------------------
#
cp $SCRIPTDIR/$SCRIPT $LOGFILE
chmod -x $OUTDIR/$LOGFILE


# Generate user input file
# ------------------------
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='A QTCM run with prescribed SST (seasonal)'
 bnddir='$BNDDIR' 
 SSTdir='$BNDDIR/SST_Reynolds'
 outdir='$OUTDIR'
 runname='$RUNNAME'
 SSTmode='real_time'
 year0= 1981
 month0= 11
 day0= 1
 lastday = 6266  ! 6266 days until 31dec1998
 noout=61
 arr3name='fvu0 coriolis part of u0 budget [m/s^2]'
 arr4name='dfu0 diffution part of u0 budget [m/s^2]'
 arr5name='adu0 advection part of u0 budget [m/s^2]'
 arr6name='dtu0 u0change part of u0 budget [m/s^2]'
 !
 ! default values for input:
 !  title='QTCM default title'          ! A decrciptive title 
 !  bnddir='../bnddata'                 ! bnd. data other than SST
 !  SSTdir='../bnddata/SST_Reynolds' ! where SST files are
 !  outdir='../proc/qtcm_output'        ! where output go
 !  runname='runname'                   ! string used to create an output filename
 !  landon=1                            ! if not 1: land is like ocean with fake 'SST'
 !  SSTmode='seasonal'                  ! decide what kind of SST to use
 !  year0 = -1                          ! starting year if <0 use year in restart file
 !  month0 = -1                         ! starting month if <0 use month in restart file
 !  day0 = -1                           ! -1 and mrestart=1: Use date saved in the restart file
 !  lastday = daysperyear               ! last day of integration
 !  interval = 1                        ! coupling interval
 !  noout=0                             ! no output for the first noout days
 !  ntout=-30                           ! monthly mean output
 !  ntouti=0                            ! monthly instantaneous data output
 !  ntoutr=-30                          ! monthly restart file
 !  mrestart=1                          ! =1: restart using qtcm.restart
 !  dt=1200.                            ! time step [seconds]
 !  mt0=1                               ! barotropic timestep every mt0 timesteps
 !  viscT=12.0e5                        ! temperature diffusion parameter [m^s/s]
 !  viscQ=12.0e5                        ! humidity diffusion parameter [m^s/s]
 !  viscU=7.0e5                         ! viscocity parameter [m^s/s]
 !  ziml=400                            ! Atmos. Mixed layer depth [m]
 !  weml=0.02                           ! Mixed layer entrainment velocity [m]
 !  Wsmin=4.5                          ! Minumum wind speed for flux calculation [m/s]
 !  V1b=-0.2204077                      ! V1 projection coeffcicient at top of mixed layer
 !  arr1name='?'                        ! Auxiliary output array names 1...8
 !  arr2name='?'                        !='?' Array not included in output.
 !  arr3name='?'
 !  arr4name='?'
 !  arr5name='?'
 !  arr6name='?'
 !  arr7name='?'
 !  arr8name='?'
 ! Example:
 ! arr1name='dps1dx Mode 1 contribution to dpsdx [m/s^2]'
 !----------------------------------------------------------------
 &end
EOF
#
# conduct ensemble experiments 
# set number of ensemble members 
# amax <=  10; for amax < 10, amax is emsemble # + 1)
#

set amax = 10
set a = 1

while ( $a < $amax )
   set fname = "0$a"
   \rm -f qtcm.restart
   cp $INIDIR/qtcm.ini_$fname qtcm.restart
   # Run qtcm using nice
   # -------------------
   # Timing the run. Output to screen and logfile
   #
   /usr/bin/time nice qtcm |& tee -a $LOGFILE

   if ($a == 1) then
      /bin/rm qm_${RUNNAME}_01.out
      cp qm_${RUNNAME}.out qm_${RUNNAME}_01.out  
   else 
      @ b = $a - 1 
      set f1 = "0$b"
      cat qm_${RUNNAME}_$f1.out qm_${RUNNAME}.out > qm_${RUNNAME}_$fname.out 
      /bin/rm qm_${RUNNAME}_$f1.out
   endif
   @ a = $a + 1
end

if ($amax == 10) then

# do the tenth ensemble
\rm -f qtcm.restart
cp $INIDIR/qtcm.ini_10 qtcm.restart
#  Run qtcm using nice; timing the run
#  Output to screen and logfile
#
/usr/bin/time nice ./qtcm |& tee -a $LOGFILE

cat qm_${RUNNAME}_09.out qm_${RUNNAME}.out > qm_${RUNNAME}_10.out
\rm qm_${RUNNAME}_09.out

endif

\rm qtcm
#
#
# ---------- end of c-shell script -------------- 
